/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_0Zj0QpMf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hkfcFKRW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hhOvOm5w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_M51yacpP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QBYRTGqT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KPeGVa5T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KBxGGb0X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XUHKD8Kb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2S7MbqUg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CMbsHRSt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_EY1ldrBK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_OWdamJds_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NyLYJQzc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_QpntpCz6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_dXUMMlbP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_nZ5nQzv4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_sBXRR3rC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_CUXKc36s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_Fnc2Yv5C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zaL3XvO1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iwIWB5cP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_L2YjaIOS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OU7QCgOV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7LGOCGxQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_43aPluLu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_n3f663Ex_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yKZqIXLb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_t0V0vN7c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_yTT9sLVV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_wJOXRYLu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_c2woNANJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_6EtgVMcJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_4P0HK28j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_TGKGDE0F_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_Z8CHK75B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_JbYmqHTV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1BumkgkH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_x31Yc0O6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_T2g8H1rX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jURBdrp6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9hRSTkgW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KHYra5Oj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_P2Y4k0wh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_V2fh4ACp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iiGixIZU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oEvP4RjQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_n2zted7h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QgopPn09_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_ghwHXRgj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QLd3tQ5M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_z6h6dMDc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Oa0Olxn5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_2qtkeMYm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_K4igXVYL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_PahQTM8c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_GE6hvAvU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_1MieGjre_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_nYiosssx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_WNb6u4pb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Sl6sfbfq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_DtRJ1JZE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_IL4KxV48_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_nRjwzNGz;
  SignalEnvelope sEnv_CUXKc36s;
  SignalEnvelope sEnv_Fnc2Yv5C;
  SignalEnvelope sEnv_Z8CHK75B;
  SignalEnvelope sEnv_JbYmqHTV;
  SignalLine sLine_Yv9PIhfZ;
  SignalLine sLine_1HxioOOy;
  ControlBinop cBinop_0Zj0QpMf;
  ControlBinop cBinop_hkfcFKRW;
  ControlBinop cBinop_hhOvOm5w;
  ControlBinop cBinop_M51yacpP;
  ControlBinop cBinop_KBxGGb0X;
  ControlBinop cBinop_2S7MbqUg;
  ControlBinop cBinop_CMbsHRSt;
  ControlBinop cBinop_EY1ldrBK;
  ControlVar cVar_OWdamJds;
  ControlVar cVar_QpntpCz6;
  SignalVarf sVarf_qQN7OlQF;
  ControlVar cVar_dXUMMlbP;
  ControlIf cIf_nZ5nQzv4;
  ControlIf cIf_sBXRR3rC;
  ControlBinop cBinop_zaL3XvO1;
  ControlBinop cBinop_L2YjaIOS;
  ControlBinop cBinop_OU7QCgOV;
  ControlBinop cBinop_n3f663Ex;
  ControlBinop cBinop_t0V0vN7c;
  SignalVarf sVarf_yW98GXPU;
  ControlVar cVar_6EtgVMcJ;
  ControlIf cIf_4P0HK28j;
  ControlIf cIf_TGKGDE0F;
  ControlBinop cBinop_1BumkgkH;
  ControlBinop cBinop_T2g8H1rX;
  ControlBinop cBinop_jURBdrp6;
  ControlBinop cBinop_P2Y4k0wh;
  ControlBinop cBinop_iiGixIZU;
  ControlVar cVar_ghwHXRgj;
  ControlBinop cBinop_QLd3tQ5M;
  SignalVarf sVarf_hZznFNgt;
  SignalVarf sVarf_t0SuIgKQ;
  SignalVarf sVarf_HG5Bxbri;
  SignalVarf sVarf_9J64Zg2X;
  ControlBinop cBinop_K4igXVYL;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
